package com.nisum.reactiveweb.exceptions;

import lombok.extern.slf4j.Slf4j;

import java.util.Map;

import org.springframework.boot.web.error.ErrorAttributeOptions;
import org.springframework.boot.web.reactive.error.DefaultErrorAttributes;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Component;
import org.springframework.web.reactive.function.server.ServerRequest;

import static java.util.Optional.ofNullable;
import static org.springframework.http.HttpStatus.BAD_REQUEST;
import static org.springframework.http.HttpStatus.INTERNAL_SERVER_ERROR;
import static org.springframework.http.HttpStatus.NOT_FOUND;

@Component
@Slf4j
public class ApiErrorAttributes extends DefaultErrorAttributes {

    @Override
    public Map<String, Object> getErrorAttributes(ServerRequest request, ErrorAttributeOptions options) {
        Map<String, Object> errorAttributes = super.getErrorAttributes(request, options);
        Throwable error = getError(request);
        log.error("{} : {}", error.getClass().getCanonicalName(), error.getMessage());
        if (error instanceof ProductNotFoundException) {
            addErrorFields(errorAttributes, error, NOT_FOUND);
        } else if (error instanceof IllegalArgumentException) {
            addErrorFields(errorAttributes, error, BAD_REQUEST);
        } else {
            int statusCode = (int) ofNullable(errorAttributes.get("status")).orElse(500);
            HttpStatus status = ofNullable(HttpStatus.resolve(statusCode)).orElse(INTERNAL_SERVER_ERROR);
            addErrorFields(errorAttributes, error, status);
        }
        return errorAttributes;
    }

    private void addErrorFields(Map<String, Object> errorAttributes, Throwable ex, HttpStatus status) {
        errorAttributes.put("message", ex.getMessage());
        errorAttributes.put("error", status);
        errorAttributes.put("status", status.value());
    }

}

